function complexity = complexity_of_series(series,weighting);
% Computes the algebraic complexity given a power series. 
% Usually called from COMPLEXITY(CONCEPT,WEIGHTING).
% weighting is 'K+1' [default]-- gives total number of literals in power series
%              'K',  simple alternative
%              'fancy' -- proportional to K but sum(w) = 0 and sum(abs(w)) = 1
%                 Note: fancy gives numbers that are difficult to interpret
%                 (eg including negative numbers), but fits the data very well.

% Series is a N x D+1 - matrix, where the last column of each polynomial is
% its degree. Here we simply add those up, with each weighted by a variable function 
% determined by "weighting."
global feature_grid;
if nargin==1, weighting = 'K+1'; end; % default
weights = [];
for degree=0:length(feature_grid)-1;
    weights = [weights; local_determine_weight(degree,weighting)];
    end;
spectrum = spectrum_of_series(series);    % spectrum is 1xD vector
if isempty(series)   % special case of empty concept and empty series
    complexity = 0; 
else
    complexity = weights' * spectrum;            % inner product of weight vector with spectrum vector
end;
%------------------------------------------------------------------------------------        
function weight = local_determine_weight(degree,weighting);
global feature_grid;
% determines weight given degree, based on second argument
% NOTE: numerous weighting schemes not yet implemented
switch weighting
    case 'K+1'               % default. Complexity is total number of literals in power_series.
        weight = degree + 1;
    case 'K'
        weight = degree;  
    case 'fancy'             % linear with constraints sum(weight) = 0; sum(abs(weight)) = 1
                             % WARNING: resulting numbers difficult to interpret
        D = length(feature_grid);
        sum_of_Ks = sum(1:D);
        half_sum = sum(1:floor(D/2));
        weight = ((degree+1)-sum_of_Ks/D)/(-2*(half_sum - floor(D/2) * sum_of_Ks/D));
    otherwise
        error('Invalid weighting scheme');
end; 

