function complexity_tool(arg);
% COMPLEXITY_TOOL: Graphical front-end for the concept algebra.
% Type 'complexity_tool' at the Matlab prompt to start.
% v0.2 by Jacob Feldman (4/04)

%-----------------------------------------------------
global current_concept;
global feature_grid;
global duality;
global h_feature_a;
global h_feature_b;
global h_feature_c;
global h_feature_d;
global h_feature_e;
global axis_handle_spectrum;
global axis_handle_grid;
global h_duality;
global h_progressive;
global progressive;
global h_output_D;
global h_output_P;
global h_output_complexity;
global h_output_significance;
global h_weighting_style;
global axis_side;
global text_fudge;
global max_spectral_power;
global weighting_style;
%---------------------------------------------------------------------------
if nargin==0
    arg='init';
end
switch arg
case 'init' %---------------------------------------------------------------
left = 0; bottom = 0;
default_weighting_style = 'K+1';  % BC-style        }
default_feature_grid = [2 2 2];   % Boolean 3-space } -> These three settings are defaults; setting
default_duality      = 'ruleout'; % = conj          }    there here replaces running ALGEBRA
h_figure = figure('Position',[0 0 1000 600],...
                  'Name','Complexity tool',...
                  'Menubar','none');
axis_handle_grid =      axes('Parent',h_figure,...
                             'Position',[-0.12 0.15 0.8 0.8],...
                             'XTick',[],'Ytick',[]); 
                         title('Concept');
axis_handle_spectrum =  axes('Parent',h_figure,...
                             'Position',[0.57 0.15 0.4 0.4]);
axes(axis_handle_grid); % makes it current for following two commands:
axis manual;
axis square;
make_feature_grid(default_feature_grid);  % installs feature grid, etc (doesn't draw grid)
duality = default_duality;
weighting_style = default_weighting_style;
set(axis_handle_grid,'ButtonDownFcn','complexity_tool grid_click');  
first_row_altitude = 50;
second_row_altitude = 20;
feature_button_left = 100;
feature_button_width = 40;
analyses_bottom = 30;
analyses_left   = 600;
analyses_width  = 160;
analyses_height = 20;
weighting_style_width = 50;
uicontrol('style','pushbutton',...
          'position',[100 first_row_altitude 80 20],...
          'string','Complement',...
          'ToolTipString','Invert current concept (Useful for creating large concepts.)',...
          'Callback','complexity_tool complement');
uicontrol('style','pushbutton',...
          'position',[20 first_row_altitude 80 20],...
          'string','Clear concept',...
          'ToolTipString','Clear current concept',...
          'Callback','complexity_tool clear');
uicontrol('style','pushbutton',...
          'position',[20 second_row_altitude 80 20],...
          'string','Analyze',...
          'ToolTipString','Analyze concept now. (Not necessary in Progressive mode.)',...
          'Callback','complexity_tool analyze');
h_duality = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + 5.5 * feature_button_width second_row_altitude 60 20],...
          'string','rule-out|rule-in',...
          'ToolTipString','Duality of algebra (See documentation.)',...
          'Callback','complexity_tool change_duality');
h_progressive = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + (5.5*feature_button_width)+60 second_row_altitude 80 20],...
          'string','batch mode|progressive',...
          'ToolTipString','Analyze continually (Progressive mode) or only when Analyze button is pressed (batch mode).',...
          'Callback','complexity_tool change_progressive');
h_weighting_style = ...
    uicontrol('style','popupmenu',...
          'position',[analyses_left - weighting_style_width analyses_bottom weighting_style_width 20],...
          'string','K+1|K|fancy',...
          'ToolTipString','Weighting style for computing complexity.',...
          'Callback','complexity_tool change_weightingstyle');
h_feature_a = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left second_row_altitude 40 20],...
          'string','2|3|4|5|6|7|8|9|10',...
          'ToolTipString','Number of levels of feature f1 (horizontal).',...
          'Callback','complexity_tool change_feature_grid');
h_feature_b = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + feature_button_width second_row_altitude 40 20],...
          'string','2|3|4|5|6|7|8|9|10',...
          'ToolTipString','Number of levels of feature f2 (vertical).',...
          'Callback','complexity_tool change_feature_grid');
h_feature_c = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + 2* feature_button_width second_row_altitude 40 20],...
          'string','-|2|3|4|5|6|7|8|9|10',...
          'ToolTipString','Number of levels of feature f3 (depth).',...
          'Callback','complexity_tool change_feature_grid');
      set(h_feature_c,'Value',2);
h_feature_d = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + 3* feature_button_width second_row_altitude 40 20],...
          'string','-|2|3|4|5|6|7|8|9|10',...
          'ToolTipString','Number of levels of feature f4 (superordinate horizontal).',...
          'Callback','complexity_tool change_feature_grid');
h_feature_e = ...
    uicontrol('style','popupmenu',...
          'position',[feature_button_left + 4* feature_button_width second_row_altitude 40 20],...
          'string','-|2|3|4|5|6|7|8|9|10',...
          'ToolTipString','Number of levels of feature f5 (superordinate vertical).',...
          'Callback','complexity_tool change_feature_grid');
uicontrol('style','text','position',[feature_button_left 5 40 13],'string','f1'); 
uicontrol('style','text','position',[feature_button_left + feature_button_width 5 40 13],'string','f2'); 
uicontrol('style','text','position',[feature_button_left + 2* feature_button_width 5 40 13],'string','f3'); 
uicontrol('style','text','position',[feature_button_left + 3* feature_button_width 5 40 13],'string','f4'); 
uicontrol('style','text','position',[feature_button_left + 4* feature_button_width 5 40 13],'string','f5'); 
h_output_D = ...
    uicontrol('style','text',...
          'position',[400 70 60 20],...
          'BackgroundColor',[1 1 1],...
          'ToolTipString','Number of dimensions in current feature grid.',...
          'string','D = ',...
          'FontSize',15); 
format long g; % sets text output (eg significance) to long format
h_output_P = ...
    uicontrol('style','text',...
          'position',[460 70 60 20],...
          'ToolTipString','Number of objects in current concept.',...
          'BackgroundColor',[1 1 1],...
          'string','P = ',...
          'FontSize',15); 
%.....................
h_output_complexity = ...
    uicontrol('style','text',...
          'position',[analyses_left analyses_bottom analyses_width analyses_height],...
          'ToolTipString','Algebraic complexity (see documentation for explanation.)',...
          'string',''); 
h_output_significance = ...
    uicontrol('style','text',...
          'position',[analyses_left + analyses_width analyses_bottom analyses_width analyses_height],...
          'ToolTipString','Probability of a concept this simple or simpler under the null hypothesis of a random concept.)',...
          'string',''); 
current_concept = [];
local_set_graphics_parameters;
local_make_grid;
case 'grid_click' %--------% handles buttondowns in the axes area------------------
local_handle_mouseclick;   
%
case 'complement' %--------% complement button ------------------------------------  
    % inverts current_concept
    universe = complete_world(feature_grid);   
    local_clear_grid; % removes all dots corresponding to (old) current_concept
    if length(current_concept) == 0 % current_concept empty
        complement = universe; 
    else                            % current_concept not empty
        complement = setdiff(universe,current_concept,'rows');
    end;
    current_concept = complement;
    for i=1:size(current_concept,1),
        local_draw_object('draw',current_concept(i,:));
    end;
    local_update_P;
    if (progressive == 2)&(length(current_concept) > 0), local_analyze; end; % update analysis NOW
case 'analyze' %--------% analyze button, or one grid buttondown if progressive ----
    local_analyze;
case 'clear' %--------% clear button -----------------------------------------------  
    % clears current_concept
local_clear_grid;
current_concept = [];
local_update_P;
axes(axis_handle_spectrum);
cla;   % clears spectrum plot
case 'change_feature_grid' % chang feature values button -----------------------------------  
% Change number of values of one feature 
n_a = get(h_feature_a,'Value')+1; % +1 because minimum choice is 2
n_b = get(h_feature_b,'Value')+1;
n_c = get(h_feature_c,'Value');
n_d = get(h_feature_d,'Value');
n_e = get(h_feature_e,'Value');  
effective_D = max(find([n_a n_b n_c n_d n_e] > 1));
    % = dimensionality, as given by highest feature with more than two levels
switch effective_D
    case 2
        appendix = [];
    case 3
        appendix = [n_c];
    case 4
        appendix = [n_c n_d];
    case 5
        appendix = [n_c n_d n_e];
end;
desired_feature_grid = [n_a n_b appendix];
make_feature_grid(desired_feature_grid);
local_set_graphics_parameters;
local_make_grid;
local_update_P;
case 'change_duality' %--------% change value of duality        ------------------
duality_index = get(h_duality,'Value');
switch duality_index
    case 1 
        duality = 'ruleout';
    case 2 
        duality = 'rulein';
end;
local_analyze;   % update analysis
case 'change_progressive' %--------% change value of progressive  ------------------
    % if progressive = 2 (ie flag ON), we analyze the concept every time an object is
    % selected or the concept is complemented. If 1 (batchmode, default), we wait for
    % the analyze button to be pressed.
progressive = get(h_progressive,'Value'); % 1 or 2
if (progressive == 2)&(length(current_concept) > 0), local_analyze; end; % update NOW!
case 'change_weightingstyle' %--------% change weighting style   ------------------
weightingstyle_index = get(h_weighting_style,'Value');
switch weightingstyle_index
    case 1
        weighting_style = 'K+1';
    case 2
        weighting_style = 'K';
    case 3
        weighting_style = 'fancy';
end;
if (progressive == 2)&(length(current_concept) > 0), local_analyze; end; % update NOW!
%---------------------------------------------------------------
    otherwise error('Bad argument to complexity server!');   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%== LOCAL FUNCTIONS ===========================================================    
function local_handle_mouseclick
% draws or erases one object at a vertex
global current_concept;
global progressive; % flag for instant analysis
cursor = get(gca,'currentpoint'); 
x = cursor(1);
y = cursor(3);  % these are the x and y coords of mouse_click
object = local_find_closest_object(x,y);
already_on = ismember(object,current_concept,'rows');
switch already_on 
    case 0      %  new object   
        current_concept = sortrows([current_concept; object]);  % add object to current_concept
        local_draw_object('draw',object);
        if (progressive == 2)&(length(current_concept) > 0), local_analyze; end; % update analysis NOW
    case 1      % already a member
        current_concept = setdiff(current_concept,object,'rows');
        local_draw_object('remove',object);
        if (progressive == 2)&(length(current_concept) > 0), local_analyze; end; % update analysis NOW
end;
local_update_P;
%---------------------------------------------
function local_draw_object(draw_or_remove,object);
% draws a dot at object on grid
% draw_or_remove is 'draw' or 'remove'
global axis_handle_grid;
global axis_side;   
axes(axis_handle_grid);
diameter = axis_side/10;  % diameter of dots at vertices
switch draw_or_remove
    case 'draw'     %...................................
        [x y] = local_vertex_location(object);
        handle = rectangle('Curvature',[1 1],...
                           'position', [x-diameter/2 y-diameter/2 diameter diameter],...
                           'FaceColor','black');
        set(handle,'UserData',object); % stores handle under UserData object
        set(handle,'ButtonDownFcn','complexity_tool grid_click');  % makes dot itself clickable
    case 'remove'   %...................................
        this_handle = findobj('UserData',object);
        delete(this_handle); drawnow;
end;
%---------------------------------------------------------------------------------------
function closest_object = local_find_closest_object(x,y);
global feature_grid;
distances = [];  
universe = complete_world(feature_grid);
for i=1:size(universe,1),
    [object_x,object_y] = local_vertex_location(universe(i,:));
    distances = [distances; norm([x,y]-[object_x,object_y])];
end;
i=1:size(universe,1);
closest_object = universe(distances(i)==min(distances),:);
%---------------------------------------------------------------------------------------
function local_clear_grid
% Removes all object dots 
% Note this function does NOT set current_concept to [], because sometimes
% it is called when we are simply clearing the grid in order to draw the
% new concept, which is not empty.
global current_concept;
global axis_handle_spectrum;
if length(current_concept) > 0,
    for i=1:size(current_concept,1)
        local_draw_object('remove',current_concept(i,:));
    end; % for
end; % if
axes(axis_handle_spectrum);
cla;   % clears spectrum plot
%---------------------------------------------------------------------------------------
function local_make_grid
% draws lines to make feature grid 
global feature_grid;
global axis_handle_grid;
global text_fudge;
global axis_side;
global margin;
global max_spectral_power;
global axis_handle_spectrum;
set(axis_handle_spectrum,'YLim',[0,max_spectral_power+1]);
axes(axis_handle_grid); % makes grid axis current
cla;  % clear axes
D = length(feature_grid);
if (D > 5) | (D < 2), error('Error: complexity_tool only works with D = 2 -- 5'); end;
num_levels_3 = 1;
num_levels_4 = 1;
num_levels_5 = 1;
if D >= 3 
    num_levels_3 = feature_grid(3);
end;
if D >= 4 
    num_levels_4 = feature_grid(4);
end;
if D == 5 
    num_levels_5 = feature_grid(5);
end;
for f5 = 0:num_levels_5-1
    for f4 = 0:num_levels_4-1
        for f3 = 0:num_levels_3-1
            appendix = [];   % appendix is values of "extra" three features
            if D==3 appendix = [f3]; end;
            if D==4 appendix = [f3 f4]; end;
            if D==5 appendix = [f3 f4 f5]; end;
            for i=0:feature_grid(1)-1
                [bottom_x bottom_y] =   local_vertex_location([i 0 appendix]);
                [top_x top_y]    =      local_vertex_location([i,feature_grid(2)-1 appendix]);
                local_drawline([bottom_x bottom_y],[top_x top_y],'complexity_tool grid_click');
                if (f3==0)&(f4==0)&(f5==0)  % labels only on bottom-left "cube":
                    text(bottom_x,bottom_y-text_fudge,num2str(i));
                end;
            end;
            for j=0:feature_grid(2)-1
                [left_x left_y] =   local_vertex_location([0 j appendix]);
                [right_x right_y] = local_vertex_location([feature_grid(1)-1 j appendix]);
                local_drawline([left_x left_y],[right_x right_y],'complexity_tool grid_click');
                if (f3==0)&(f4==0)&(f5==0)  % labels only on bottom-left "cube":
                    text(left_x-text_fudge,left_y,num2str(j));
                end;
            end;
            if (f3==0)&(f4==0)&(f5==0)      % draw feature legend on bottom-left "cube":
                [bottom_x bottom_y] =   local_vertex_location([0 0 appendix]);
                text(bottom_x+axis_side/2,bottom_y-2*text_fudge,'f1');
                text(bottom_x-2*text_fudge,bottom_y+axis_side/2,'f2');
            end;
            if D>2          % draw a line connecting up the 3rd dim
                [left_x left_y]     =   local_vertex_location([0 feature_grid(2)-1 0 appendix(2:end)]);
                [right_x right_y]   =   local_vertex_location([0 feature_grid(2)-1 feature_grid(3)-1 appendix(2:end)]);
                local_drawline([left_x left_y],[right_x right_y],'complexity_tool grid_click'); 
                if (f4==0)&(f5==0)&(num_levels_3>1)
                    for k=0:num_levels_3-1
                        [bottom_right_x bottom_right_y] = ...
                            local_vertex_location([feature_grid(1)-1 0 k appendix(2:end)]);
                        label_string = ['f3=' num2str(k)];
                        text(bottom_right_x + text_fudge,bottom_right_y+text_fudge,label_string);
                    end; % for
                end; % if
            end;
        end; % end individual "cube"
    end;
end;
%......Draw superordinate feature labels:
switch D
    case 4
        for f4=0:feature_grid(4)-1
            [left y]  = local_vertex_location([0 0 0 f4]);
            [right y] = local_vertex_location([feature_grid(1)-1 0 0 f4]);
            text((left+right)/2,margin/2-text_fudge,['f4=' num2str(f4)]);
        end;
    case 5
        for f4=0:feature_grid(4)-1
            [left y]  = local_vertex_location([0 0 0 f4 0]);
            [right y] = local_vertex_location([feature_grid(1)-1 0 0 f4 0]);
            text((left+right)/2,margin/2-text_fudge,['f4=' num2str(f4)]);
        end;
        for f5=0:feature_grid(5)-1
            [left bottom]  = local_vertex_location([0 0 0 0 f5]);
            [left top]     = local_vertex_location([0 feature_grid(2)-1 0 0 f5]);
            text(margin/2-2*text_fudge,(top+bottom)/2-2*text_fudge,['f5=' num2str(f5)]);
        end;
end;
local_update_D;
%---------------------------------------------------------------------------------------
function [x,y] = local_vertex_location(object);
% Gives screen location in GUI of object.
global feature_grid;
global axis_side;
global axes_fudge;
global margin;
global third_dimension_angle;
global super_dimension_side; % grid size for 4th and 5th dimension
D = length(feature_grid);
if (D > 5) | (D < 2), error('Error: complexity_tool only works with D = 2--5'); end;
a = object(1); 
b = object(2);
c = 0;
d = 0;
e = 0;
if D >= 3, 
    c = object(3); 
end;
if D >=4
    d = object(4);
end;
if D == 5
    e = object(5);
end;   % now a, b, c, d, and e are effective 5 values -- even when D = 2, etc.
x = margin + a * axis_side/(feature_grid(1)-1) ...
    + c * axes_fudge * axis_side * cos(third_dimension_angle) ...
    + d * super_dimension_side;     % Fourth dimension goes ACROSS the screen "superordinately"
y = margin + b * axis_side/(feature_grid(2)-1) ...
    + c * axes_fudge * axis_side * sin(third_dimension_angle) ...
    + e * super_dimension_side;     % Fifth dimension goes UP the screen "superordinately"
%---------------------------------------------------------------------------
function local_update_D; 
%  Changes D output field
global h_output_D;
global feature_grid;
D = length(feature_grid);
new_string = strcat(['D = ' num2str(D)]);
set(h_output_D,'string',new_string);
%---------------------------------------------------------------------------
function local_update_P; 
%  Changes P output field
global h_output_P;
global current_concept;
P= size(current_concept,1);
new_string = strcat(['P = ' num2str(P)]);
set(h_output_P,'string',new_string);
%---------------------------------------------------------------------------
function significance = local_compute_significance(complexity);
% Computes significance based on formulae in Feldman "Quantifying
% Eureka...", Cognition, 2003.
% Note these formulae only work for binary concepts, hence the conditional.
% This function should only be fed complexity computing using 'K+1' weighting_style
global feature_grid;
global duality;
if strcmp(duality,'ruleout')
if exist('normcdf')==2 % ---- stats package installed
    if max(feature_grid) > 2
        significance = -1; % error flag
    else
    D = length(feature_grid);
    alpha = 0.36287;   % These quantities and formulae are from Feldman (2003)
    beta = .86686;
    mu_D = alpha * exp(beta * D);
    z = (complexity - mu_D)/sqrt(mu_D);
    significance = normcdf(z,0,1); 
    end;
else
    significance=-1;    % stats package not installed: set flag to error
end;
else
    significance=-1;    % duality rulein-- significance unknown
end;
%---------------------------------------------------------------------------
function local_analyze;
% Perform analysis and plot results on or near spectrum axis
global axis_handle_spectrum;
global current_concept;
global feature_grid;
global h_output_complexity;
global h_output_significance;
global max_spectral_power;
global weighting_style;
D = length(feature_grid);
series = power_series(current_concept);
spectrum = spectrum_of_series(series);   
complexity = complexity_of_series(series,weighting_style);
axes(axis_handle_spectrum);     % makes spectrum axis current
if ~isempty(spectrum)
    bar(0:D-1,spectrum);        % Plots spectrum
    ylim('manual');
    ylim([0 max_spectral_power+1]); % sets ylimits to standard for feature_grid
    ylabel('Power (# regularities)');
    xlabel('Algebraic degree (K)');
    title('Power spectrum');
end;
% ......................... % Update complexity:
complexity_string = strcat(['Algebraic complexity: ' num2str(complexity)]);
set(h_output_complexity,'string',complexity_string);
% ......................... % Update significance:
significance = local_compute_significance(complexity_of_series(series,'K+1'));
  % Note significance is computed based on k+1-style complexity, regardless
  % of what weighting style we're actually using.
if significance > -1
    significance_string = strcat(['Significance: ' num2str(significance)]);
else
    significance_string = '';  % Nonbinary concept; significance unknown.
end;
set(h_output_significance,'string',significance_string);

%---------------------------------------------------------------------------------------
function local_set_graphics_parameters
%  Sets various parameters that are need in local_vertex_location and
%  local_make_grid
global axis_side;
global feature_grid;
global axes_fudge;
global margin;
global third_dimension_angle;
global super_dimension_side;
global text_fudge;
global max_spectral_power;   % estimated max on y-axis
% Compute geometry of grids:
binary_first_two_features = (feature_grid(1)==2)&(feature_grid(2)==2);
if binary_first_two_features  % determines separation between levels of 3rd dimension
    axes_fudge = .71; % here they overlap;
else
    axes_fudge = 1.5; % here they don't.
end; % if
margin = 0.1; % room on all sides of grid
third_dimension_angle = pi/6;  % angle at which 3rd dimension sticks out; default 60 degs
D = length(feature_grid);  
if D>2 max_features_3=feature_grid(3); else max_features_3 = 1; end;
%.....Determine scaledown of grid box:
switch D
    case 2
        scale_down = 1;
    case 3 
        scale_down = 1;
    case 4
        scale_down = feature_grid(4);
    case 5
        scale_down = max(feature_grid(4),feature_grid(5));
end;
super_dimension_side = (1-2*margin)/scale_down;
axis_side = super_dimension_side/(axes_fudge * (max_features_3+1));
text_fudge = axis_side/10;  % makes the label text miss the dots
max_spectral_power = round(prod(feature_grid)/2);   % will be used to limit y-axis of spectrum
%---------------------------------------------------------------------------------------
function local_drawline(a,b,button_function) % a and b points, ie 1x2 vectors
% draw segment from a to b, and sets ButtonDownFcn
line = [a;b];
hold on;   % because usually you want to add a line to the current plot
handle = plot(line(:,1),line(:,2));
set(handle,'ButtonDownFcn',button_function)
