function polynomial_list = power_series(concept,desired_duality,desired_feature_grid)
% POWER_SERIES computes the power series of concept, using feature_grid and duality
% If these are not given, uses previously set global values.
% If these are not previously set, sets them to defaults.

%----------------------------------------------------------------------------------------
% Initialize:
global feature_grid;            % either previously set or will be set here
global duality;                 % set to 'ruleout' or 'rulein'( = respectively conj or disj)
%----------------------------------------------------------------------------------------
% First, set feature_grid and duality to given values, if provided.
switch nargin
    case 3
        make_feature_grid(desired_feature_grid);
        duality = desired_duality;
    case 2
        duality = desired_duality;
    case 1;
        % do nothing -- duality and feature_grid will be inherited or set to
        % defaults.
end; % switch nargin
%----------------------------------------------------------------------------------------
% Next, make sure duality and feature_grid are set:
if isempty(duality)             % ie not previously set
    duality = 'ruleout';        % default duality
end;
if isempty(feature_grid)&isempty(concept)
    feature_grid = [2 2];       % fallback default if everything is empty;
    display('Assuming feature grid: ');
    display(feature_grid);
end;
if isempty(feature_grid)        % ie not previously set
    default_feature_grid = [];  % procedure for setting default feature_grid based on the input concept:
    for j = 1:size(concept,2)
        highest_feature_value = max(concept(:,j));
        default_feature_grid = [default_feature_grid max([highest_feature_value+1 2])]; % set to min necessary, or else 2
    end; % for
    display('Assuming feature grid: ');
    make_feature_grid(default_feature_grid);
    display(feature_grid);
end; % if
global regularity_sequence;     % should have been previously set by make_feature_grid
universe = complete_world(feature_grid); % set of ALL objects
%candidate_polynomial = zeros(1,length(feature_grid)+1)-1;  % row of -1's; 
                % this is the first-tried polyonmial and seeds the rest of
                % them in private_next_polynomial;
polynomials = regularity_sequence;   % global, previously set by make_feature_grid
    % all regularities, in lex (K) order; K is last (D+1-th) field
regularity_index = 2;   % which regularity are we trying? Start from 2 because first is all -1s and thus undefined.
model_complete = 0;     % flag for having completely constructed the correct model
switch duality
    case 'ruleout'
        initial_model = complete_world(feature_grid);
    case 'rulein'
        initial_model = [];
    otherwise
        error('Invalid duality');
end;
current_model = initial_model;             % initialize set of objects genererated by current theory
current_theory = [];                       % current candidate set of polynomials
%------------------------------------------------------------
if (size(concept,2) ~= length(feature_grid)) & ~isempty(concept),
    error('Concept does not match feature grid.');  
else
excess_check = [];
for i=1:size(concept,1)
    excess_check = [excess_check; concept(i,:) >= feature_grid];
end;
if  max(excess_check)
    error('Concept has at least one feature out of bounds of feature_grid.');
else
% ............. Main loop..................................
while ~model_complete
    candidate_polynomial = polynomials(regularity_index,:);
    if private_correct_and_nontrivial(concept,current_theory,current_model,candidate_polynomial),
        current_theory = [current_theory; candidate_polynomial]; %
        current_model = private_theory2model(current_theory);  % update current_model
    end;
    model_complete = ...
        isequal(sortrows(current_model),sortrows(concept)) || (isempty(concept) && isempty(current_model));
    regularity_index = regularity_index + 1; % note start from #2 (since #1 is undefined)
end; % while loop
%..........................................................
polynomial_list = current_theory;
end; % if-clause with warning at top
end