function both = private_correct_and_nontrivial(world,old_theory,old_model,polynomial) 
% Tests whether the given polynomial is both correct and nontrivial
% of World, given duality and current feature_grid (both global)
% Correct means:
%  ruleout: all objects don't violate polynomial
%  rulein:  objects are a superset of polynomial
% Nontrivial means:
%  new model differs from old model.
global feature_grid;
global duality;
%----------------------------------------------------------------
new_model = private_theory2model([old_theory;polynomial]);
%---------------------------------------
% nontrivial:
nontrivial = ~isequal(sortrows(old_model),sortrows(new_model)); % new model differs from old
%---------------------------------------
% correct:
new_part_of_model = private_theory2model(polynomial); % note this will depend on duality
switch duality
    case 'ruleout'
        objects_obey_new_part = [];  
        for i=1:size(world,1),
            objects_obey_new_part = [objects_obey_new_part; ~private_within_shaded_area(polynomial,world(i,:))];
        end;  % if new polynomial is correct, object_obeys_new_part should be all ones.
%     NOTE we could have done this with ismember, but we are trying to be
%     efficient by only looking at the new part of the model rather than
%     the whole new model!
    case 'rulein'
       objects_obey_new_part = ismember(new_part_of_model,world,'rows'); % here, it's simpler
end;
if isempty(objects_obey_new_part)
    correct = 1;                     % if there are no objects, they automatically "obey the new part"
else
    correct = min(objects_obey_new_part);
end;
both = correct & nontrivial;
%--------------------------------------- 