function sequence = private_regularity_sequence(feature_grid);
% gives matrix of regularities given feature_grid,
% in lexicographic order
% with K is last (D+1-th) field
% A regularity is a vector of values, where the value -1 means "don't care". 
%....................
% NOTE this function is compatible with both "conj" and "disj" duality
% but the "semantics" of a regularity is different in each.
% The regularity simply defines a "shaded area" in the feature space;
% In "conj" duality this means "no objects here [and..]", but in "disj"
% it means                     "objects allowed here [or ...].
%----------------------------------------------------
D = length(feature_grid);
unsorted_reg_set = private_complete_world(feature_grid+1)-1;
nonneg_matrix = unsorted_reg_set > -1;  % matrix with 1s and 0's
Ks = sum(nonneg_matrix,2)-1;   % sum along rows to get K
regs_with_Ks = [unsorted_reg_set Ks];
sequence = sortrows(regs_with_Ks,D+1);
%  NOTE: first line of this is all -1s (undefined regularity)
