function model = private_theory2model(theory)
% Given feature_grid and duality computes objects that satisfy polynomials in theory
global feature_grid;
global duality;
%---------------------------------
universe = complete_world(feature_grid);
model_indices = [];
switch duality
    case 'ruleout'       % aka conj  
        for i=1:size(universe,1)
            object = universe(i,:);
            if object_obeys_all_polynomials(object,theory)
                model_indices = [model_indices;i]; 
            end;
        end;
    case 'rulein'       % aka disj
        for i=1:size(universe,1)
            object = universe(i,:);
            if object_obeys_one_polynomial(object,theory)
                model_indices = [model_indices;i]; 
            end;
        end;
end;
model = universe(model_indices,:);

%--------------------------------------------------------------------------
function object_obeys_all_polynomials = object_obeys_all_polynomials(object,theory)
% true if object obeys [ruleout] EVERY polynomial in theory
% note this subfunction only makes sense in RULEOUT duality
obeys = [];
for i=1:size(theory,1);  % klugy for-loop
    obeys = [obeys; ~private_within_shaded_area(theory(i,:),object)]; % NOT in this shaded area
end;
object_obeys_all_polynomials = min(obeys);
%--------------------------------------------------------------------------
function object_obeys_one_polynomial = object_obeys_one_polynomial(object,theory)
% true if object obeys [rulein] ONE polynomial in theory 
% note this subfunction only makes sense in RULEIN duality
obeys = [];
for i=1:size(theory,1);  % klugy for-loop
    obeys = [obeys; private_within_shaded_area(theory(i,:),object)]; % IS in this shaded area
end;
object_obeys_one_polynomial = max(obeys);
