function spectrum = spectrum_of_series(series);
% Computes spectrum from a power series. 
% The spectrum is the function giving the number of implication polynomials
% at each degree. The output is 1xD vector of integers.

%-------
% Note: a power series has the degree in the last column of each row
% (polynomial). Note also that the spectrum itself does not depend on
% the weigting scheme; that's only used to compute complexity.
global feature_grid;
rownumbers = 1:size(series,1);
if isempty(series)
    spectrum = zeros(size(feature_grid));  % return empty spectrum for empty series
else
    last_column = series(1:size(series,1),length(feature_grid)+1);
    spectrum = [];
    for K=0:length(feature_grid)-1,
        % K is algebraic degree
        spectrum = [spectrum; sum(last_column==K)]; % append number of elements in last column that match degree
    end;
end